package com.qmth.distributed.print.business.util;

import com.aspose.words.*;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.config.DictionaryConfig;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import javax.annotation.Resource;
import java.io.*;
import java.util.Objects;

/**
 * @Description: aspose pdf util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/7
 */
@Component
public class AsposePdfUtil {
    private final static Logger log = LoggerFactory.getLogger(AsposePdfUtil.class);

    @Resource
    DictionaryConfig dictionaryConfig;

    /**
     * 获取license
     *
     * @return
     */
    private boolean getLicense() {
        boolean result = false;
        try {
            File license = null;
            License aposeLic = new License();
            if (dictionaryConfig.sysDomain().getDocumentToPdfLicense().startsWith(ResourceUtils.CLASSPATH_URL_PREFIX)) {
                license = ResourceUtils.getFile(dictionaryConfig.sysDomain().getDocumentToPdfLicense());
                aposeLic.setLicense(new FileInputStream(license));
            } else {
                InputStream stream = getClass().getClassLoader().getResourceAsStream(dictionaryConfig.sysDomain().getDocumentToPdfLicense());//classes下的读取方法
//                license = new File(dictionaryConfig.sysDomain().getWordToPdfLicense());//放服务器上的读取方法
                aposeLic.setLicense(stream);
            }
            result = true;
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return result;
    }

    /**
     * document转pdf
     *
     * @param documentPath
     * @param pdfPath
     * @return
     * @throws IOException
     */
    public File documentToPdf(String documentPath, String pdfPath, int paperSize) throws IOException {
        if (!getLicense()) { // 验证License 若不验证则转化出的pdf文档会有水印产生
            throw ExceptionResultEnum.ERROR.exception("hyml转pdf失败，验证过期");
        }
        FileOutputStream os = null;
        File file = null;
        try {
            long start = System.currentTimeMillis();
            file = new File(pdfPath); //新建一个pdf文档
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
                file.createNewFile();
            }
            os = new FileOutputStream(file);
            Document doc = new Document(documentPath); //Address是将要被转化的word文档
            DocumentBuilder builder = new DocumentBuilder(doc);
            PageSetup pageSetup = builder.getPageSetup();
            pageSetup.setPaperSize(paperSize);
            pageSetup.setVerticalAlignment(PageVerticalAlignment.TOP);
            if (paperSize == PaperSize.A3) {
                pageSetup.setOrientation(Orientation.PORTRAIT);
            }
            doc.save(os, com.aspose.words.SaveFormat.PDF);//全面支持DOC, DOCX, OOXML, RTF HTML, OpenDocument, PDF, EPUB, XPS, SWF 相互转换
//            ossUtil.ossUpload(pdfPath, new FileInputStream(file), DigestUtils.md5Hex(new FileInputStream(file)));
            long end = System.currentTimeMillis();
            log.info("共耗时：" + ((end - start) / 1000.0) + "秒"); //转化用时
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            if (Objects.nonNull(os)) {
                os.flush();
                os.close();
            }
        }
        return file;
    }
}
